"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const medusa_1 = require("@medusajs/medusa");
const axios_1 = __importDefault(require("axios"));
const format_registration_name_1 = __importDefault(require("@medusajs/medusa/dist/utils/format-registration-name"));
const types_1 = require("../types");
const utils_1 = require("../utils");
class SentryService extends medusa_1.TransactionBaseService {
    constructor({ manager, eventBusService }, config) {
        // @ts-ignore
        super(...arguments);
        this.sentryApiBaseUrl = 'https://sentry.io/api/0/organizations';
        this.manager_ = manager;
        this.config_ = config;
        this.eventBusService_ = eventBusService;
    }
    /**
     * Fetch paginated transactions from an organisation project on sentry
     * @param organisation The organisation on which to fetch the transactions
     * @param project The project in the organisation on which to fetch the transactions
     * @param statsPeriod The period from when to fetch the transactions (default: 24h)
     * @param perPage The number of transaction per page
     * @param token The token to use to send request to sentry
     * @param cursor The cursor to send to fetch the transactions for a given page
     * @return The result is composed of the data and the next cursor for the pagination purpose
     */
    async fetchSentryTransactions({ organisation, project, statsPeriod, perPage, token, cursor, }) {
        perPage = perPage !== null && perPage !== void 0 ? perPage : 100;
        const queryParams = {
            field: 'transaction',
            per_page: Number(perPage),
            project,
            query: `event.type:transaction`,
            statsPeriod,
            // The three values from cursor are: cursor identifier (integer, usually 0), row offset, and is_prev (1 or 0).
            // e.g 0:10:0
            cursor,
        };
        return await this.fetchSentry({
            organisation,
            token,
            perPage,
            queryParams,
        });
    }
    /**
     * Fetch paginated transaction events from an organisation project on sentry
     * @param transaction The transaction for which to fetch the events (e.g "GET /admin/users")
     * @param organisation The organisation on which to fetch the transactions
     * @param project The project in the organisation on which to fetch the transactions
     * @param statsPeriod The period from when to fetch the transactions (default: 24h)
     * @param perPage The number of transaction per page
     * @param token The token to use to send request to sentry
     * @param cursor The cursor to send to fetch the transactions for a given page
     * @return The result is composed of the data and the next cursor for the pagination purpose
     */
    async fetchTransactionEvents({ transaction, organisation, project, statsPeriod, perPage, token, cursor, }) {
        perPage = perPage !== null && perPage !== void 0 ? perPage : 100;
        const queryParams = {
            field: ['id', 'transaction.duration', 'timestamp', 'spans.db'],
            per_page: Number(perPage),
            project,
            query: `event.type:transaction transaction:"${transaction}"`,
            statsPeriod,
            sort: '-timestamp',
            // The three values from cursor are: cursor identifier (integer, usually 0), row offset, and is_prev (1 or 0).
            // e.g 0:10:0
            cursor,
        };
        return await this.fetchSentry({
            organisation,
            token,
            perPage,
            queryParams,
        });
    }
    async handleIssues(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnIssue)) {
                return await this.config_.webHookOptions.emitOnIssue(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_ISSUE, data);
        });
    }
    async handleErrors(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnError)) {
                return await this.config_.webHookOptions.emitOnError(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_ERROR, data);
        });
    }
    async handleComments(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnComment)) {
                return await this.config_.webHookOptions.emitOnComment(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_COMMENT, data);
        });
    }
    async handleAlerts(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnEventOrMetricAlert)) {
                return await this.config_.webHookOptions.emitOnEventOrMetricAlert(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_EVENT_OR_METRIC_ALERT, data);
        });
    }
    async handleInstallation(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnInstallOrDeleted)) {
                return await this.config_.webHookOptions.emitOnInstallOrDeleted(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_INSTALL_OR_DELETED, data);
        });
    }
    async fetchSentry({ organisation, token, queryParams, perPage }) {
        const url = this.sentryApiBaseUrl + `/${organisation}/events/`;
        const searchParams = new URLSearchParams();
        Object.entries(queryParams).forEach(([key, value]) => {
            if (Array.isArray(value)) {
                value.forEach((val) => {
                    searchParams.append(key, val);
                });
            }
            else {
                value && searchParams.append(key, value === null || value === void 0 ? void 0 : value.toString());
            }
        });
        const { data: { data, meta }, headers, } = await axios_1.default.get(url, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
            params: searchParams,
        });
        const nextCursor = SentryService.buildNextCursor(headers['link']);
        let prevCursor;
        if (nextCursor) {
            prevCursor = SentryService.buildPrevCursor(nextCursor, perPage);
        }
        return { data, meta, prev_cursor: prevCursor, next_cursor: nextCursor };
    }
    static buildNextCursor(link) {
        var _a;
        let result = '';
        const nextCursorMatch = link === null || link === void 0 ? void 0 : link.match(/.*cursor="(.*)"$/);
        if (nextCursorMatch && nextCursorMatch[1]) {
            result = (_a = nextCursorMatch[1]) === null || _a === void 0 ? void 0 : _a.split(',')[0];
        }
        return result;
    }
    static buildPrevCursor(nextCursor, perPage) {
        const parts = nextCursor.split(':');
        const prevCursorItems = Math.max(0, Number(parts[1]) - perPage * 2);
        return `${parts[0]}:${prevCursorItems}:${parts[2]}`;
    }
}
exports.default = SentryService;
SentryService.RESOLVE_KEY = (0, format_registration_name_1.default)(`${process.cwd()}/services/sentry.js`);
//# sourceMappingURL=sentry.js.map